﻿/******************************************************************
*        PoLAR (Portable Library for Augmented Reality)           *
*     Copyright (C) 2015 Inria, Université de Lorraine, CNRS      *
*                                                                 *
* This library is free software; you can redistribute it and/or   *
* modify it under the terms of the GNU General Public License     *
* as published by the Free Software Foundation; either version 3  *
* of the License, or (at your option) any later version.          *
*                                                                 *
* This library is distributed in the hope that it will be useful, *
* but WITHOUT ANY WARRANTY; without even the implied warranty of  *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the    *
* GNU General Public License for more details.                    *
*                                                                 *
*                                                                 *
* You should have received a copy of the GNU General Public       *
* License along with this library.                                *
* If not, see <http://www.gnu.org/licenses/>.                     *
******************************************************************/


#include <QColorDialog>
#include "ARBallonInterface.h"
#include "props.h"
//#include "EpipolarPickHandler.h"

using namespace PoLAR;

Interface::Interface(): QMainWindow()
{
    QString dir = dataPath() + "icons/";

    // Create a Viewer with shadows management
    viewer = new PoLAR::Viewer(this, "viewer", 0, true);
    viewer->addEventHandler(new osgViewer::StatsHandler);// add stats handler

    //Ajout du sol pour les ombres
    osg::ref_ptr<osg::Geode> sol = new osg::Geode;
    osg::ref_ptr<osg::ShapeDrawable> shape;
    shape = new osg::ShapeDrawable(new osg::Box(osg::Vec3(5.5f, 4.0f, 0.0f), 10.2f, 6.0f, 0.00001f));
    sol->addDrawable(shape.get());
    osg::ref_ptr<PoLAR::Object3D> object3D = new PoLAR::Object3D(sol.get());
    object3D->setName("sol");
    object3D->setPhantomOn();
    viewer->addObject3D(object3D.get());
    object3D->setEditOn();

    //ajout de la source de lumière
    viewer->addLightSource(2,2,-5, true);
    viewer->show();

    _pick = new PoLAR::TabBoxPickHandler; // create pick handler
    _pick->setCharacterSize(15);
    viewer->addEventHandler(_pick);// add pick handler

    PoLAR::DefaultPickHandler *geomPick = new PoLAR::DefaultPickHandler;

    geomPick->setCharacterSize(15);
    viewer->addEventHandler(geomPick);// add geometry pick handler
    toolbar = new QToolBar(this);
    addToolBar(Qt::LeftToolBarArea, toolbar);
    toolbar->setOrientation(Qt::Vertical);

    scrollbarTB = new QToolBar(this);
    scrollbarTB->setOrientation(Qt::Vertical);

    scrollbar = new QScrollBar(this);
    scrollbar->setMinimum(0);
    scrollbar->setMaximum(0);
    scrollbar->setPageStep(10);
    scrollbar->setValue(0);
    scrollbar->setFixedHeight(height());
    scrollbar->setOrientation(Qt::Vertical);
    scrollbarTB->addWidget(scrollbar);


    QObject::connect(scrollbar, SIGNAL(valueChanged(int)), viewer, SLOT(gotoImageSlot(int)));
    QObject::connect(scrollbar, SIGNAL(valueChanged(int)), this, SLOT(changeTitle(int)));

    scrollbarTB->update();
    addToolBar(Qt::RightToolBarArea, scrollbarTB);

    move(0,0);

    QObject::connect(this, SIGNAL(startEditMarkersSignal()), viewer, SLOT(startEditMarkersSlot()));


    /******************************************************************/
    /*                    Création de l'interface                     */
    /******************************************************************/

    QWidget *widgetInit = new QWidget();
    layoutInit = new QGridLayout(widgetInit);
    toolbar->addWidget(widgetInit);

    //Selection de la côte
    QLabel *nObj = new QLabel("Objet :");
    layoutInit->addWidget(nObj, 1, 0);

    combo = new QComboBox(this);
    combo->addItem("Ballon");
    layoutInit->addWidget(combo, 1, 1);
    index=0;
    connect(combo, SIGNAL(currentIndexChanged(int)),this, SLOT(setIndex(int)));



    //Création du slider poour faire pivoter une côte selon l'angle 1

    QLabel *labAngle1 = new QLabel("Angle 1 :");
    layoutInit->addWidget(labAngle1, 2, 0);
    QSlider *sliderAngle1 = new QSlider(Qt::Horizontal);
    sliderAngle1->setRange(1, 11);
    sliderAngle1->setValue(6);
    layoutInit->addWidget(sliderAngle1, 2, 1, 1, 3);
    angle1prec = 6;

    //Quand la valeur du slider angle1 change, on appel la fonction rotationcoteAngle1
    connect(sliderAngle1, SIGNAL(valueChanged(int)),this, SLOT(rotationcoteAngle1(int)));


    //Création du slider poour faire pivoter une côte selon l'angle 2

    QLabel *labAngle2 = new QLabel("Angle 2 :");
    layoutInit->addWidget(labAngle2, 3, 0);

    QSlider *sliderAngle2 = new QSlider(Qt::Horizontal);
    sliderAngle2->setRange(1, 11);
    sliderAngle2->setValue(6);
    layoutInit->addWidget(sliderAngle2, 3, 1, 1, 3);
    angle2prec = 6;

    connect(sliderAngle2, SIGNAL(valueChanged(int)),this, SLOT(rotationcoteAngle2(int)));

    QCheckBox *checkboxVisible = new QCheckBox("Visible", this);
    checkboxVisible->setChecked(true);
    layoutInit->addWidget(checkboxVisible, 4, 0);

    connect(checkboxVisible, SIGNAL(clicked(bool)),this, SLOT(visible(bool)));

    QCheckBox *checkboxMesh= new QCheckBox("Mesh", this);
    layoutInit->addWidget(checkboxMesh, 4, 1);

    connect(checkboxMesh, SIGNAL(clicked(bool)),this, SLOT(mesh(bool)));


    // add focus to this interface in order to get the keyboard events
    this->setFocusPolicy(Qt::StrongFocus);
    // remove the focus from the Viewer, otherwise it will bypass the interface's events
    viewer->setFocusPolicy(Qt::NoFocus);
}

Interface::~Interface()
{
    delete lightDialog;
    delete mb;
}

void Interface::activateLabelBox(Marker2D *m)
{
    labelBox->setEnabled(true);
    if (m) labelBox->setValue((int)m->label());
}

void Interface::deactivateLabelBox()
{
    labelBox->setValue(-1);
    labelBox->setEnabled(false);
}

void Interface::changeTitle(int n)
{
    char buf[256];
    sprintf(buf, "Image %3d",n);
    statuslabel->setText(*(new QString(buf)));
}

void Interface::saveMarkers()
{
    QString fname = QFileDialog::getSaveFileName( this,
                                                  "Choose a filename to save the markers",
                                                  QString::null);
    if (!fname.isEmpty()) viewer->saveObject2D(fname);

}

void Interface::mesh(bool state)
{
    osg::ref_ptr<PoLAR::Object3D> obj =   viewer->getObject3D(index);
    if(state==true)
    {
        // Pour représenter le maillage et non la géométrie
         obj->getOrCreateStateSet()->setAttributeAndModes(
         new osg::PolygonMode(osg::PolygonMode::FRONT_AND_BACK,
         osg::PolygonMode::LINE),  osg::StateAttribute::OVERRIDE);
    }
    else
    {
        // Pour représenter la géométrie et non le maillage
         obj->getOrCreateStateSet()->setAttributeAndModes(
         new osg::PolygonMode(osg::PolygonMode::FRONT_AND_BACK,
         osg::PolygonMode::FILL),  osg::StateAttribute::OVERRIDE);
    }
}

void Interface::pickedObjectInRed(bool state)
{
    osg::ref_ptr<PoLAR::Object3D> obj =   viewer->getObject3D(index);
    if(state==true)
    {
        // Pour colorier en rouge
         if (obj.valid()) obj->setAmbientColor(1.0,0.0,0.0);
    }
    else
    {
        // Pour colorier à nouveau en blanc
         if (obj.valid()) obj->setAmbientColor(0.0,0.0,0.0);
    }
}

//Affiche ou non l'objet en fonction de la checkboxVisible
void Interface::visible(bool state)
{
    osg::ref_ptr<PoLAR::Object3D> obj =   viewer->getObject3D(index);
    if(state==true && obj->isDisplayed()==false)
    {
        obj->setDisplayOn();
    }

    else
    {
        if(state==false && obj->isDisplayed()==true)
        {
            obj->setDisplayOff();
        }
    }
}


//Met l'index à l'entier correspondant à l'objet seclectionné dans le menu deroulant et remet les sliders à leurs valeurs min
void Interface::setIndex(int i)
{
    index=i+1;
}


//Fait pivoter une cote en fonction de la modification de la position du SLider
void Interface::rotationcoteAngle1(int i)
{

    osg::ref_ptr<PoLAR::Object3D> obj =   viewer->getObject3D(index);

    if(angle1prec<i)
    {
        /*osg::Vec3d tmp = obj->getOrigin();
        tmpX = tmp.osg::Vec3d::x();
        tmpY = tmp.osg::Vec3d::y();
        tmpZ = tmp.osg::Vec3d::z();*/
        obj->rotate(M_PI/50,0,1,0);
        //obj->translate(tmpX/2,tmpY/2,0);
        angle1prec=i;
    }

    else
    {
        /*osg::Vec3d tmp = obj->getOrigin();
        tmpX = tmp.osg::Vec3d::x();
        tmpY = tmp.osg::Vec3d::y();
        tmpZ = tmp.osg::Vec3d::z();*/
        obj->rotate(-M_PI/50,0,1,0);
        //obj->translate(tmp);
        angle1prec=i;
    }
}


//Fait pivoter une cote en fonction de la modification de la position du SLider
void Interface::rotationcoteAngle2(int i)
{

    osg::ref_ptr<PoLAR::Object3D> obj =   viewer->getObject3D(index);

    if(angle2prec<i)
    {
        obj->rotate(M_PI/50,0,0,1);
        angle2prec=i;
    }

    else
    {
        obj->rotate(-M_PI/50,0,0,1);
        angle2prec=i;
    }
}

// Add key bindings
void Interface::keyPressEvent(QKeyEvent *event)
{
    switch(event->key())
    {
    case Qt::Key_Q: this->exit(); break;
    case Qt::Key_D:
    {
        // Hide/display background image
        viewer->bgImageToggle();
        break;
    }
    case Qt::Key_W:
    {
        // Set the scene graph interaction to be active
        viewer->startManipulateSceneSlot();
        break;
    }
    case Qt::Key_G:
    {
        QString fname=QFileDialog::getSaveFileName( this,
                                                    "Choose a filename to save the snapshot image",
                                                    QString::null,
                                                    "PNG images (*.png)");
        if (!fname.isEmpty()) viewer->grab().save(fname, "PNG");

        break;
    }
    default:
        viewer->notify(event);
    }
}

void Interface::keyReleaseEvent(QKeyEvent *event)
{
    //simply forwarding to the viewer
    viewer->notify(event);
}
